body {
  font-family: Helvetica,Arial,sans-serif;
  margin: 0;
  background-color: #fff;
  color: #000;
}

i, em {
  font-family: Times New Roman,Times,serif;
}

a:link { 
  color: #730;
}

a:visited { 
  color: #402;
}

a:hover {
  background-color: #fd8;
}

div#header {
  display: block;
  background-color: #930;
  color: #fd6;
  border-bottom: 3px solid #f70;
  padding: 3px;
  font-size: 30px;
}

div#header h1 { 
  padding: 0;
  margin: 0;
  font-size: 1.5em;
}

div#nav {
  float: left;
  background-color: #fd9;
  border: 1px solid #f70;
  margin-right: 1em;
  border-bottom: 1px solid #f70;
  width: 200px;
}

div#nav ul {
  padding: 0;
  margin: 0;
}

div#nav li {
  list-style: none;
  margin: 0;
}

div#nav ul li ul li a { 
  padding: 2px 2em 2px 2em;
  font-weight: normal;
}

div#nav a {
  display: block;
  padding: 2px 1em 2px 1em;
  text-decoration: none;
  color: #400;
  font-weight: bold;
}

div#nav a:hover {
  background-color: #f80;
  color: #fff;
}

/* If I ever do menus that show the "current" page, that would be
   by marking some "links" as selected */
div#nav span.selected {
  display: block;
  font-weight: bold;
  padding: 2px 1em 2px 1em;
}

div#body {
  padding: 1em;
}

h1.page-title {
  margin-top: 0;
}